@@ -3,7 +3,7 @@ ldapobject.py - wraps class _ldap.LDAPObject
 
 See http://www.python-ldap.org/ for details.
 
-\$Id: ldapobject.py,v 1.137 2013/11/23 15:44:32 stroeder Exp $
+\$Id: ldapobject.py,v 1.141 2014/09/09 10:14:47 stroeder Exp $
 
 Compability:
 - Tested with Python 2.0+ but should work with Python 1.5.x
@@ -41,6 +41,13 @@ from ldap.extop import ExtendedRequest,ExtendedResponse
 from ldap import LDAPError
 
 
+class NO_UNIQUE_ENTRY(ldap.NO_SUCH_OBJECT):
+  """
+  Exception raised if a LDAP search returned more than entry entry
+  although assumed to return a unique single search result.
+  """
+
+
 class SimpleLDAPObject:
   """
   Drop-in wrapper class around _ldap.LDAPObject
@@ -638,24 +645,59 @@ class SimpleLDAPObject:
     except IndexError:
       return None
 
+  def read_s(self,dn,filterstr=None,attrlist=None,serverctrls=None,clientctrls=None,timeout=-1):
+    """
+    Reads and returns a single entry specified by `dn'.
+    
+    Other attributes just like those passed to `search_ext_s()'
+    """
+    r = self.search_ext_s(
+      dn,
+      ldap.SCOPE_BASE,
+      filterstr or '(objectClass=*)',
+      attrlist=attrlist,
+      serverctrls=serverctrls,
+      clientctrls=clientctrls,
+      timeout=timeout,
+    )
+    if r:
+      return r[0][1]
+    else:
+      return ldap.NO_SUCH_OBJECT('Empty search result reading %s' % (repr(dn)))
+
   def read_subschemasubentry_s(self,subschemasubentry_dn,attrs=None):
     """
     Returns the sub schema sub entry's data
     """
-    attrs = attrs or SCHEMA_ATTRS
     try:
-      r = self.search_s(
-        subschemasubentry_dn,ldap.SCOPE_BASE,
-        '(objectClass=subschema)',
-        attrs
+      subschemasubentry = self.read_s(
+        subschemasubentry_dn,
+        filterstr='(objectClass=subschema)',
+        attrlist=attrs or SCHEMA_ATTRS
       )
     except ldap.NO_SUCH_OBJECT:
       return None
     else:
-      if r:
-        return r[0][1]
-      else:
-        return None
+      return subschemasubentry
+
+  def find_unique_entry(self,base,scope=ldap.SCOPE_SUBTREE,filterstr='(objectClass=*)',attrlist=None,attrsonly=0,serverctrls=None,clientctrls=None,timeout=-1):
+    """
+    Returns a unique entry, raises exception if not unique
+    """
+    r = self.search_ext_s(
+      base,
+      scope,
+      filterstr,
+      attrlist=attrlist or ['*'],
+      attrsonly=attrsonly,
+      serverctrls=serverctrls,
+      clientctrls=clientctrls,
+      timeout=timeout,
+      sizelimit=2,
+    )
+    if len(r)!=1:
+      raise NO_UNIQUE_ENTRY('No or non-unique search result for %s' % (repr(filterstr)))
+    return r[0]
 
 
 class NonblockingLDAPObject(SimpleLDAPObject):
@@ -729,7 +771,7 @@ class ReconnectLDAPObject(SimpleLDAPObject):
     self._options = []
     self._last_bind = None
     SimpleLDAPObject.__init__(self,uri,trace_level,trace_file,trace_stack_limit)
-    self._reconnect_lock = self._ldap_lock(desc='reconnect')
+    self._reconnect_lock = ldap.LDAPLock(desc='reconnect lock within %s' % (repr(self)))
     self._retry_max = retry_max
     self._retry_delay = retry_delay
     self._start_tls = 0
@@ -773,9 +815,10 @@ class ReconnectLDAPObject(SimpleLDAPObject):
     try:
       reconnect_counter = retry_max
       while reconnect_counter:
+        counter_text = '%d. (of %d)' % (retry_max-reconnect_counter+1,retry_max)
         if __debug__ and self._trace_level>=1:
-          self._trace_file.write('*** Trying %d. (of %d) reconnect to %s...\n' % (
-            retry_max-reconnect_counter+1,retry_max,uri
+          self._trace_file.write('*** Trying %s reconnect to %s...\n' % (
+            counter_text,uri
           ))
         try:
           # Do the connect
@@ -783,17 +826,17 @@ class ReconnectLDAPObject(SimpleLDAPObject):
           self._restore_options()
           # StartTLS extended operation in case this was called before
           if self._start_tls:
-            self.start_tls_s()
+            SimpleLDAPObject.start_tls_s(self)
           # Repeat last simple or SASL bind
           self._apply_last_bind()
         except (ldap.SERVER_DOWN,ldap.TIMEOUT),e:
           if __debug__ and self._trace_level>=1:
-            self._trace_file.write('*** %d. (of %d) reconnect to %s failed\n' % (
-              self._retry_max-reconnect_counter+1,retry_max,uri
+            self._trace_file.write('*** %s reconnect to %s failed\n' % (
+              counter_text,uri
             ))
           reconnect_counter = reconnect_counter-1
           if not reconnect_counter:
-            raise
+            raise e
           if __debug__ and self._trace_level>=1:
             self._trace_file.write('=> delay %s...\n' % (retry_delay))
           time.sleep(retry_delay)
@@ -801,8 +844,8 @@ class ReconnectLDAPObject(SimpleLDAPObject):
           del self._l
         else:
           if __debug__ and self._trace_level>=1:
-            self._trace_file.write('*** %d. (of %d) reconnect to %s successful, last operation will be repeated\n' % (
-              self._retry_max-reconnect_counter+1,retry_max,uri
+            self._trace_file.write('*** %s reconnect to %s successful => repeat last operation\n' % (
+              counter_text,uri
             ))
           self._reconnects_done = self._reconnects_done + 1L
           break
